@@ -16,7 +16,7 @@ use Term::ANSIColor qw/colored/;
 use File::ShareDir qw/dist_dir/;
 use Path::Tiny;
 
-our $VERSION = "0.002";
+our $VERSION = "0.003";
 
 has [qw/base_url api_key api_pass test/] => (
     is => 'rw',
@@ -59,29 +59,26 @@ sub list {
     my ($self, $opt, $query) = @_;
     my $jenkins = $self->jenkins();
 
-    my $data = $jenkins->_json_api([qw/api json/], { extra_params => { depth => 1 } });
-
-    for my $job (sort @{ $data->{jobs} }) {
-        next if $query && $job->{name} !~ /$query/;
-        my $name = $job->{name};
+    $self->_action(1, $query, sub {
+        my $name = $_->{name};
         my $extra = '';
 
-        if ( $job->{color} =~ s/_anime// ) {
+        if ( $_->{color} =~ s/_anime// ) {
             $extra = '*';
         }
 
         if ( $opt->{verbose} ) {
             eval {
-                my $details = $jenkins->_json_api(['job', $job->{name}, qw/api json/], { extra_params => { depth => 1 } });
+                my $details = $jenkins->_json_api(['job', $_->{name}, qw/api json/], { extra_params => { depth => 1 } });
                 $extra .= "\t" . localtime( ( $details->{lastBuild}{timestamp} || 0 ) / 1000 );
             };
         }
 
         # map "jenkins" colours to real colours
-        my $color = $self->colour_map->{$job->{color}} || [$job->{color}];
+        my $color = $self->colour_map->{$_->{color}} || [$_->{color}];
 
         print colored($color, $name), " $extra\n";
-    }
+    });
 
     return;
 }
@@ -190,7 +187,9 @@ sub create {
     my ($self, $opt, $job, $config, @extra) = @_;
     my $jenkins = $self->jenkins();
 
-    $jenkins->create_job($job, $config);
+    my $success = $jenkins->create_job($job, $config);
+
+    print $success ? "Created $job\n" : "Error creating $job\n";
 
     return;
 }
@@ -244,31 +243,47 @@ sub _xslt_actions {
     my $data = $jenkins->_json_api([qw/api json/], { extra_params => { depth => 0 } });
 
     my %found;
-    for my $job (sort @{ $data->{jobs} }) {
-        next if $query && $job->{name} !~ /$query/;
+    $self->_action(0, $query, sub {
 
-        my $config = $jenkins->project_config($job->{name});
+        my $config = $jenkins->project_config($_->{name});
         my $dom = XML::LibXML->load_xml(string => $config);
 
         my $results = $stylesheet->transform($dom);
         my $output  = $stylesheet->output_as_bytes($results);
 
-        warn "Updating $job->{name}\n" if $opt->{verbose};
+        warn "Updating $_->{name}\n" if $opt->{verbose};
         if ($opt->{test}) {
             print "$output\n";
         }
         else {
-            my $success = $jenkins->set_project_config($job->{name}, $output);
+            my $success = $jenkins->set_project_config($_->{name}, $output);
             if (!$success) {
-                warn "Error in updating $job->{name}\n";
+                warn "Error in updating $_->{name}\n";
                 last;
             }
         }
+    });
+
+    return;
+}
+
+sub _action {
+    my ($self, $depth, $query, $action) = @_;
+    my $jenkins = $self->jenkins();
+
+    my $data = $jenkins->_json_api([qw/api json/], { extra_params => { depth => $depth } });
+
+    for my $job (sort @{ $data->{jobs} }) {
+        next if $query && $job->{name} !~ /$query/;
+        local $_ = $job;
+
+        $action->();
     }
 
     return;
 }
 
+
 1;
 
 __END__
